@@ -23,17 +23,31 @@ Import the content of a mbox file into the database.
 Author: Aurelien Bompard <abompard@fedoraproject.org>
 """
 
+from __future__ import absolute_import
+
 import mailbox
 import os
 import re
 import urllib
+import sys
+import logging
 from dateutil.parser import parse
 from dateutil import tz
 from optparse import OptionParser
 from random import randint
 from email.utils import unquote
+from urllib2 import HTTPError
+from traceback import print_exc
 
+from mailman.interfaces.archiver import ArchivePolicy
+from storm.exceptions import DatabaseError
+from mailmanclient import MailmanConnectionError
+from kittystore import SchemaUpgradeNeeded
 from kittystore.scripts import get_store_from_options, StoreFromOptionsError
+from kittystore.utils import get_mailman_client
+from kittystore.caching import sync_mailman
+from kittystore.search import make_delayed
+from kittystore.test import FakeList
 
 
 PREFIX_RE = re.compile("^\[([\w\s_-]+)\] ")
@@ -74,26 +88,31 @@ U(?:rl|RL)[ ]?:[ ]([^\s]+)\s*\n
 TEXTWRAP_RE = re.compile("\n\s*")
 
 
-def convert_date(date_string):
-    """ Convert the string of the date to a datetime object. """
-    #print date_string
-    date_string = date_string.split('(')[0].strip()
-    dt = parse(date_string)
-    return dt.astimezone(tz.tzutc())
-
-
-class DummyMailingList(object):
-    # pylint: disable=R0903
-    # (Too few public methods)
-    def __init__(self, address):
-        self.fqdn_listname = unicode(address)
-        self.display_name = None
-        self.subject_prefix = None
+def awarify(date):
+    if date.tzinfo is None or date.tzinfo.utcoffset(date) is None:
+        return date.replace(tzinfo=tz.tzutc())
+    return date
 
 
 class DownloadError(Exception): pass
 
 
+def get_mailinglist(list_name, settings, opts):
+    mlist = FakeList(list_name)
+    try:
+        mm_client = get_mailman_client(settings)
+        mm_list = mm_client.get_list(list_name)
+    except (HTTPError, MailmanConnectionError), e:
+        if opts.debug:
+            print "Can't get the mailing-list from Mailman: %s" % e
+    else:
+        mlist_settings = mm_list.settings
+        mlist.display_name = mlist_settings["display_name"]
+        mlist.subject_prefix = mlist_settings["subject_prefix"]
+        mlist.archive_policy = getattr(ArchivePolicy, mlist_settings["archive_policy"])
+    return mlist
+
+
 class DbImporter(object):
     """
     Import email messages into the KittyStore database using its API.
@@ -106,6 +125,13 @@ class DbImporter(object):
         self.force_import = opts.duplicates
         self.no_download = opts.no_download
         self.verbose = opts.verbose
+        self.since = opts.since
+        if opts.cont:
+            self.since = store.get_last_date(self.mlist.fqdn_listname)
+        if self.since is not None:
+            self.since = awarify(self.since)
+            if self.verbose:
+                print "Only emails after %s will be imported" % self.since
 
     def from_mbox(self, mbfile):
         """ Upload all the emails in a mbox file into the database using
@@ -115,20 +141,35 @@ class DbImporter(object):
         upload to the database.
         :arg list_name, the fully qualified list name.
         """
+        self.store.search_index = make_delayed(self.store.search_index)
         cnt_imported = 0
         cnt_read = 0
         for message in mailbox.mbox(mbfile):
+            if self.since:
+                date = message["date"]
+                if date:
+                    try:
+                        date = awarify(parse(date))
+                    except ValueError, e:
+                        print "Can't parse date string in message %s: %s" \
+                              % (message["message-id"], date)
+                        print e
+                        continue
+                    if date < self.since:
+                        continue
             cnt_read = cnt_read + 1
             self.total_imported += 1
+            if self.verbose:
+                print "%s (%d)" % (message["Message-Id"], self.total_imported)
             # Un-wrap the subject line if necessary
             if message["subject"]:
                 message.replace_header("subject",
                         TEXTWRAP_RE.sub(" ", message["subject"]))
             # Try to find the mailing-list subject prefix in the first email
-            if cnt_read == 1:
+            if not self.mlist.subject_prefix and message["subject"]:
                 subject_prefix = PREFIX_RE.search(message["subject"])
                 if subject_prefix:
-                    self.mlist.display_name = unicode(subject_prefix.group(1))
+                    self.mlist.subject_prefix = unicode(subject_prefix.group(1))
             if self.force_import:
                 while self.store.is_message_in_list(
                             self.mlist.fqdn_listname,
@@ -155,6 +196,12 @@ class DbImporter(object):
                 print "%s from %s about %s" % (e.args[0],
                         e.args[1].get("From"), e.args[1].get("Subject"))
                 continue
+            except DatabaseError:
+                print_exc()
+                print ("Message %s failed to import, skipping"
+                       % unquote(message["Message-Id"]))
+                self.store.rollback()
+                continue
             # And insert the attachments
             for counter, att in enumerate(attachments):
                 self.store.add_attachment(
@@ -164,7 +211,9 @@ class DbImporter(object):
 
             self.store.flush()
             cnt_imported += 1
-        self.store.commit()
+            # Commit every time to be able to rollback on error
+            self.store.commit()
+        self.store.search_index.flush() # Now commit to the search index
         if self.verbose:
             print '  %s email read' % cnt_read
             print '  %s email added to the database' % cnt_imported
@@ -222,15 +271,15 @@ class DbImporter(object):
 def parse_args():
     usage = "%prog -s store_url -l list_name mbox_file [mbox_file ...]"
     parser = OptionParser(usage=usage)
-    parser.add_option("-s", "--store", help="the URL to the store database")
     parser.add_option("-l", "--list-name", help="the fully-qualified list "
             "name (including the '@' symbol and the domain name")
-    parser.add_option("-i", "--search-index", metavar="PATH",
-                      help="the path to the search index")
-    parser.add_option("--settings",
+    parser.add_option("-s", "--settings", default="settings",
                       help="the Python path to a Django settings module")
-    parser.add_option("--pythonpath",
+    parser.add_option("-p", "--pythonpath",
                       help="a directory to add to the Python path")
+    parser.add_option("-c", "--continue", action="store_true", dest="cont",
+                      help="only import newer emails")
+    parser.add_option("--since", help="only import emails after this date")
     parser.add_option("-v", "--verbose", action="store_true",
             help="show more output")
     parser.add_option("-d", "--debug", action="store_true",
@@ -240,6 +289,8 @@ def parse_args():
     parser.add_option("-D", "--duplicates", action="store_true",
             help="do not skip duplicate emails (same Message-ID header), "
                  "import them with a different Message-ID")
+    parser.add_option("--no-sync-mailman", action="store_true",
+            help="don't sync with Mailman after importing")
     opts, args = parser.parse_args()
     if opts.list_name is None:
         parser.error("the list name must be given on the command-line.")
@@ -251,17 +302,33 @@ def parse_args():
     for mbfile in args:
         if not os.path.exists(mbfile):
             parser.error("No such mbox file: %s" % mbfile)
+    if opts.since is not None:
+        if opts.cont:
+            parser.error("--since and --continue are mutually exclusive")
+        try:
+            opts.since = parse(opts.since)
+        except ValueError, e:
+            parser.error("invalid value for '--since': %s" % e)
     try:
         store = get_store_from_options(opts)
     except StoreFromOptionsError, e:
         parser.error(e.args[0])
+    except SchemaUpgradeNeeded:
+        print >>sys.stderr, ("The database schema needs to be upgraded, "
+                             "please run kittystore-updatedb first")
+        sys.exit(1)
     return store, opts, args
 
 
 def main():
     store, opts, args = parse_args()
+    if opts.debug:
+        debuglevel = logging.DEBUG
+    else:
+        debuglevel = logging.INFO
+    logging.basicConfig(format='%(message)s', level=debuglevel)
     print 'Importing messages from %s to database...' % opts.list_name
-    mlist = DummyMailingList(opts.list_name)
+    mlist = get_mailinglist(opts.list_name, store.settings, opts)
     importer = DbImporter(mlist, store, opts)
     for mbfile in args:
         print "Importing from mbox file %s" % mbfile
@@ -269,3 +336,6 @@ def main():
         if opts.verbose:
             print '  %s emails are stored into the database' \
                   % store.get_list_size(opts.list_name)
+    if not opts.no_sync_mailman:
+        sync_mailman(store)
+    store.commit()
