@@ -27,9 +27,12 @@ from __future__ import absolute_import
 
 import importlib
 import sys
+import logging
 from optparse import OptionParser
 
-from kittystore import get_store
+from kittystore import get_store, create_store, SchemaUpgradeNeeded
+
+from kittystore.caching import sync_mailman
 
 
 #
@@ -37,36 +40,31 @@ from kittystore import get_store
 #
 class StoreFromOptionsError(Exception): pass
 
-def get_store_from_options(opts):
+def get_settings_from_options(opts):
     """
-    Returns a Store instance from an options object. Known options are;
-    - "store": the store URL
-    - "search_index": the search index path
+    Returns a settings object from an options object. Known options are;
     - "settings": the Django settings module
     - "pythonpath": an additional Python path to import the Django settings
     """
-    django_settings = None
-    if opts.settings is not None:
-        if opts.pythonpath is not None:
-            sys.path.append(opts.pythonpath)
-        try:
-            django_settings = importlib.import_module(opts.settings)
-        except ImportError as e:
-            raise StoreFromOptionsError(
-                    "could not import settings '%s' (Is it on "
-                    "sys.path?): %s" % (opts.settings, e))
-    if opts.store is not None:
-        store_url = opts.store
-    elif getattr(django_settings, "KITTYSTORE_URL", None) is not None:
-        store_url = django_settings.KITTYSTORE_URL
-    else:
+    settings = None
+    if opts.pythonpath is not None:
+        sys.path.append(opts.pythonpath)
+    try:
+        settings = importlib.import_module(opts.settings)
+    except ImportError as e:
         raise StoreFromOptionsError(
-                "you must either specify a store URL (eg: "
-                "sqlite:///kittystore.sqlite) or a Django configuration "
-                "module (Python path to the settings module)")
-    if opts.search_index is None:
-        opts.search_index = getattr(django_settings, "KITTYSTORE_SEARCH_INDEX", None)
-    return get_store(store_url, search=opts.search_index, debug=opts.debug)
+                "could not import settings '%s' (Is it on "
+                "sys.path?): %s" % (opts.settings, e))
+    return settings
+
+def get_store_from_options(opts):
+    """
+    Returns a Store instance from an options object. Known options are;
+    - "settings": the Django settings module
+    - "pythonpath": an additional Python path to import the Django settings
+    """
+    settings = get_settings_from_options(opts)
+    return get_store(settings, debug=opts.debug)
 
 
 #
@@ -74,31 +72,79 @@ def get_store_from_options(opts):
 #
 
 def updatedb():
-    parser = OptionParser(usage="%prog -s store_url")
-    parser.add_option("-s", "--store", metavar="URL",
-                      help="the URL to the store database")
-    parser.add_option("-i", "--search-index", metavar="PATH",
-                      help="the path to the search index")
-    parser.add_option("--settings",
-                      help="the Python path to a Django settings module")
-    parser.add_option("--pythonpath",
+    parser = OptionParser(usage="%prog -s settings_module")
+    parser.add_option("-s", "--settings", default="settings",
+                      help="the Python path to a Django-like settings module")
+    parser.add_option("-p", "--pythonpath",
                       help="a directory to add to the Python path")
     parser.add_option("-d", "--debug", action="store_true",
                       help="show SQL queries")
     opts, args = parser.parse_args()
     if args:
         parser.error("no arguments allowed.")
+    if opts.debug:
+        debuglevel = logging.DEBUG
+    else:
+        debuglevel = logging.INFO
+    logging.basicConfig(format='%(message)s', level=debuglevel)
     print 'Upgrading the database schema and populating ' \
           'the search index if necessary...'
     try:
-        store = get_store_from_options(opts)
-    except StoreFromOptionsError, e:
+        settings = get_settings_from_options(opts)
+    except (StoreFromOptionsError, AttributeError), e:
         parser.error(e.args[0])
-    version = list(store.db.execute(
-                "SELECT patch.version FROM patch "
-                "ORDER BY version DESC LIMIT 1"
-                ))[0][0]
-    print "Done, the current schema version is %d." % version
+    try:
+        store = get_store(settings, debug=opts.debug)
+    except SchemaUpgradeNeeded:
+        print "Upgrading the schema..."
+        store = create_store(settings, debug=opts.debug)
+        version = list(store.db.execute(
+                    "SELECT patch.version FROM patch "
+                    "ORDER BY version DESC LIMIT 1"
+                    ))[0][0]
+        print "Done, the current schema version is %d." % version
+        print "Synchonizing data from Mailman, this can take some time..."
+        sync_mailman(store)
+        store.commit()
+        print "  ...done!"
+    else:
+        print "No schema upgrade needed."
+
+
+
+#
+# Manual Mailman sync
+#
+
+def sync_mailman_cmd():
+    parser = OptionParser(usage="%prog -s settings_module [-f]")
+    parser.add_option("-s", "--settings", default="settings",
+                      help="the Python path to a Django-like settings module")
+    parser.add_option("-p", "--pythonpath",
+                      help="a directory to add to the Python path")
+    parser.add_option("-d", "--debug", action="store_true",
+                      help="show SQL queries")
+    opts, args = parser.parse_args()
+    if args:
+        parser.error("no arguments allowed.")
+    if opts.debug:
+        debuglevel = logging.DEBUG
+    else:
+        debuglevel = logging.INFO
+    logging.basicConfig(format='%(message)s', level=debuglevel)
+    print 'Synchronizing data from Mailman...'
+    try:
+        store = get_store_from_options(opts)
+    except (StoreFromOptionsError, AttributeError), e:
+        parser.error(e.args[0])
+    except SchemaUpgradeNeeded:
+        print >>sys.stderr, ("The database schema needs to be upgraded, "
+                             "please run kittystore-updatedb first")
+        sys.exit(1)
+    sync_mailman(store)
+    store.commit()
+    print "  ...done!"
+
 
 
 #
