@@ -23,8 +23,6 @@ import binascii
 from types import IntType
 from mimetypes import guess_all_extensions
 
-from email.utils import unquote
-
 from mailman.utilities.string import oneline
 
 # Path characters for common platforms
@@ -201,7 +199,7 @@ class Scrubber(object):
                     t = unicode(t, partcharset, 'replace')
                 except (UnicodeError, LookupError, ValueError,
                         AssertionError):
-                    # We can get here if partcharset is bogus in come way.
+                    # We can get here if partcharset is bogus in some way.
                     # Replace funny characters.  We use errors='replace'
                     t = unicode(t, 'ascii', 'replace')
                 # Separation is useful
@@ -210,10 +208,14 @@ class Scrubber(object):
                         t += '\n'
                     text.append(t)
 
-            text = "\n".join(text)
+            text = u"\n".join(text)
         else:
             text = self.msg.get_payload(decode=True)
-            text = text.decode(get_charset(self.msg, guess=True), "replace")
+            charset = get_charset(self.msg, guess=True)
+            try:
+                text = text.decode(charset, "replace")
+            except (UnicodeError, LookupError, ValueError, AssertionError):
+                text = unicode(text, 'ascii', 'replace')
 
             next_part_match = NEXT_PART.search(text)
             if next_part_match:
@@ -234,7 +236,7 @@ class Scrubber(object):
         # i18n file name is encoded
         try:
             filename = oneline(part.get_filename(''), in_unicode=True)
-        except TypeError:
+        except (TypeError, UnicodeDecodeError):
             # Workaround for https://bugs.launchpad.net/mailman/+bug/1060951
             # (accented filenames)
             filename = "attachment.bin"
