@@ -5,7 +5,6 @@ import unittest
 import email
 import datetime
 import dateutil
-from mock import Mock
 
 from mailman.email.message import Message
 
@@ -72,8 +71,8 @@ class TestUtils(unittest.TestCase):
         msg["References"] = " "
         msg.set_payload("Dummy message")
         try:
-            ref_id = kittystore.utils.get_ref(msg)
-        except IndexError, e:
+            kittystore.utils.get_ref(msg)
+        except IndexError:
             self.fail("Empty 'References' tag should be handled")
 
     def test_non_ascii_headers(self):
@@ -93,6 +92,23 @@ class TestUtils(unittest.TestCase):
             self.assertEqual(h_out, h_expected)
             self.assertTrue(isinstance(h_out, unicode))
 
+    def test_bad_header(self):
+        """
+        utils.header_to_unicode must handle badly encoded non-ascii headers
+        """
+        testdata = [
+            ("Guillermo G\xf3mez", u"Guillermo G\ufffdmez"),
+            ("=?gb2312?B?UmU6IFJlOl9bQW1iYXNzYWRvcnNdX01hdGVyaWFfc29icmVfb19DRVNvTF8oRGnhcmlvX2RlX2JvcmRvKQ==?=",
+                u"Re: Re:_[Ambassadors]_Materia_sobre_o_CESoL_(Di\ufffdrio_de_bordo)"),
+        ]
+        for h_in, h_expected in testdata:
+            try:
+                h_out = kittystore.utils.header_to_unicode(h_in)
+            except UnicodeDecodeError, e:
+                self.fail(e)
+            self.assertEqual(h_out, h_expected)
+            self.assertTrue(isinstance(h_out, unicode))
+
     def test_wrong_datestring(self):
         datestring = "Fri, 5 Dec 2003 11:41 +0000 (GMT Standard Time)"
         parsed = kittystore.utils.parsedate(datestring)
@@ -122,3 +138,13 @@ class TestUtils(unittest.TestCase):
         header = "=?x-gbk?Q?Frank_B=A8=B9ttner?="
         decoded = kittystore.utils.header_to_unicode(header)
         self.assertEqual(decoded, u'Frank B\ufffd\ufffdttner')
+
+    def test_no_from(self):
+        msg = Message()
+        msg.set_payload("Dummy message")
+        try:
+            name, email = kittystore.utils.parseaddr(msg["From"])
+        except AttributeError, e:
+            self.fail(e)
+        self.assertEqual(name, '')
+        self.assertEqual(email, '')
